from leak import logger


def versions_split(version_str, type_applyer=int):
    dots_count = version_str.count('.')
    if dots_count == 0:
        major, minor, patch = version_str, 0, 0
    elif dots_count == 1:
        major, minor = version_str.split('.')
        patch = 0
    elif dots_count == 2:
        major, minor, patch = version_str.split('.')
    else:
        logger.debug(
            'Incorrect version "{version}". '
            'Move to bottom when sorting'.format(version=version_str))
        major, minor, patch = 0, 0, 0

    return list(map(type_applyer, (major, minor, patch)))



import logging
import datetime


logging.basicConfig()
logger = logging.getLogger(__package__)


DATE_FORMAT = '%d/%m/%Y %H:%M'
EPOCH_BEGIN = datetime.datetime.fromtimestamp(0)
FIRST_COLUMN_LENGTH = 20



import argparse

from leak import main


def cli():
    parser = argparse.ArgumentParser(
            description='Show all releases for package and some info about it')
    parser.add_argument('package_name',
                        help='You should provide package name')
    args = parser.parse_args()
    main.main(package_name=args.package_name)


if __name__ == '__main__':
    cli()



import os

from setuptools import setup
from setuptools.command.test import test as TestCommand


install_requires = [
    'requests',
    'termcolor',
    'packaging',
]
tests_require = install_requires + [
    'pytest==3.1.2',
    'tox==2.7.0',
    'mock==2.0.0',
]
release_require = [
    'zest.releaser==6.12',
    'twine>=1.9.1,<2.0',
]


def read(filepath):
    with open(os.path.join(os.path.dirname(__file__), filepath)) as f:
        return f.read().strip()


class PyTest(TestCommand):
    user_options = []

    def run(self):
        import subprocess
        import sys
        errno = subprocess.call([sys.executable, '-m', 'pytest', 'tests'])
        raise SystemExit(errno)


args = dict(
    name='leak',
    version='1.2.0',
    description='Show available releases for package',
    long_description=read('README.rst'),
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Version Control',
        'Topic :: Utilities',
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        ],
    author='Misha Behersky',
    author_email='bmwant@gmail.com',
    url='http://bmwlog.pp.ua/post/117',
    license='MIT License',
    packages=['leak'],
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require={
        'test': tests_require,
        'dev': tests_require + release_require,
    },
    scripts=['scripts/leak'],
    include_package_data=True,
    cmdclass=dict(test=PyTest)
)

setup(**args)



import datetime
import functools

import requests

from packaging.version import parse as parse_version

from termcolor import colored

from leak import logger
from leak import EPOCH_BEGIN, FIRST_COLUMN_LENGTH, DATE_FORMAT
from leak.version_parser import versions_split


def get_latest_time_for_release(release):
    latest_time = EPOCH_BEGIN
    date_format = '%Y-%m-%dT%H:%M:%S'
    for release_record in release:
        upload_time = datetime.datetime.strptime(
            release_record['upload_time'], date_format
        )
        if upload_time > latest_time:
            latest_time = upload_time
    return latest_time


def get_max_downloads_for_release(release):
    max_downloads = 0
    for release_record in release:
        if release_record['downloads'] > max_downloads:
            max_downloads = release_record['downloads']
    return max_downloads


def show_package_info(data):
    def print_row(caption_text, value):
        caption = caption_text.ljust(FIRST_COLUMN_LENGTH)
        print('%s %s' % (caption, value))

    info = data['info']
    name = info['name']
    summary = info['summary']
    author = info['author']
    author_email = info['author_email']
    url = info['home_page']
    versions_count = len(data['releases'])

    print('='*80)

    header = colored(name, 'white')
    print(header)
    print(summary)

    print('-'*80)

    print_row('Author:', author)
    print_row('Author mail:', author_email)
    print_row('Available versions:', versions_count)
    print_row('Home page:', url)

    print('='*80)


def get_package_data(package_name):
    url_template = 'https://pypi.org/pypi/{package_name}/json'
    url = url_template.format(package_name=package_name)
    resp = requests.get(url)
    if resp.status_code != 200:
        raise ValueError('No such package')

    data = resp.json()
    return data


def parse_packages_from_html(html_content):
    return ''


def search_for_package(package_name):
    url_template = ('https://pypi.python.org/'
                    'pypi?:action=search&term={package_name}')
    url = url_template.format(package_name=package_name)
    resp = requests.get(url)
    if resp.status_code != 200:
        return []
    return parse_packages_from_html(resp.text)


def main(package_name=''):
    try:
        package_data = get_package_data(package_name)
    except ValueError as e:
        error_text = colored('No such package "%s"' % package_name, 'red')
        print(error_text)
        return

    releases = package_data['releases']
    show_package_info(package_data)
    most_popular_count = 0
    most_popular_release = None
    most_recent_release = None
    most_recent_date = EPOCH_BEGIN

    try:
        versions = sorted(releases.keys(), reverse=True, key=parse_version)
    except ValueError as e:
        logger.debug('Trying to sort versions as strings')
        splitter = functools.partial(versions_split, type_applyer=str)
        versions = sorted(releases.keys(), reverse=True, key=splitter)

    for release_num, release_data in releases.items():
        downloads_count = get_max_downloads_for_release(release_data)
        if downloads_count > most_popular_count:
            most_popular_count = downloads_count
            most_popular_release = release_num

        upload_date = get_latest_time_for_release(release_data)
        if upload_date > most_recent_date:
            most_recent_date = upload_date
            most_recent_release = release_num

    for version in versions:
        if version == most_popular_release == most_recent_release:
            text = colored('(%s) Most popular and recent. Use it' % version, 'green')
            print(text)
        elif version == most_popular_release:
            text = colored('(%s) Most popular. %s downloads' % (version, most_popular_count), 'yellow')
            print(text)
        elif version == most_recent_release:
            text = colored('(%s) Most recent. %s release date' % (version, most_recent_date.strftime(DATE_FORMAT)), 'blue')
            print(text)
        else:
            text = '(%s)' % version
            print(text)
